/*!
 * @brief     API configuration header file.
 *            This file contains API configuration parameters.
 *
 * @copyright copyright(c) 2018 analog devices, inc. all rights reserved.
 *            This software is proprietary to Analog Devices, Inc. and its
 *            licensor. By using this software you agree to the terms of the
 *            associated analog devices software license agreement.
 */

/*!
 * @addtogroup ADI_API_CONFIG
 * @{
 */
#ifndef __ADI_CMS_API_CONFIG_H__
#define __ADI_CMS_API_CONFIG_H__

/*============= D E F I N E S ==============*/
#define ADI_REPORT_VERBOSE    ADI_CMS_LOG_ALL   /*!< verbose log report control */
#define ADI_INVALID_POINTER   0                 /*!< invalid pointer address    */

#endif /* __ADI_API_CONFIG_H__ */
/*! @} */